# Onirix Camera Controls Module


![Version](https://img.shields.io/badge/version-1.2.1-blue.svg?cacheSeconds=2592000)
[![Documentation](https://img.shields.io/badge/documentation-yes-brightgreen.svg)](https://docs.onirix.com/tutorials/camera-controller)
[![Twitter: onirix](https://img.shields.io/twitter/follow/onirix.svg?style=social)](https://twitter.com/onirix)

This library is an addon to [Onirix Embed SDK](https://www.npmjs.com/package/@onirix/embed-sdk) that allows manipulating
the camera in a more intuitive and direct way to perform effects such as:
- Camera animations when entering a scene.
- Moving the camera to different points of interests around the scene.
- Advanced camera controls, e.g., zoom customization using interface elements.
- Restricting camera movement in some ways.

Full documentation available at <a href="https://docs.onirix.com/tutorials/camera-controller" target="_blank">Onirix Docs</a>.

## Install

```sh
npm install @onirix/embed-sdk
npm install @onirix/camera-controls-module
```

Or include the dependency with the HTML head tag:

```html
<head>
    <script src="https://cdn.jsdelivr.net/npm/@onirix/embed-sdk@1.17.1/+esm"></script>
    <script src="https://cdn.jsdelivr.net/npm/@onirix/camera-controls-module@1.2.1/+esm"></script>
</head>
```

Or as ESM modules:

```js
import OnirixEmbedSDK from 'https://cdn.jsdelivr.net/npm/@onirix/embed-sdk@1.17.1/+esm';
import OnirixCameraModule from 'https://cdn.jsdelivr.net/npm/@onirix/camera-controls-module@1.2.1/+esm';
```

## Usage

### Initialization
Before accessing any of its functions, the camera module must be initialized by providing an instance of the
[Onirix Embed SDK](https://www.npmjs.com/package/@onirix/embed-sdk) as shown below:

```js
import OnirixEmbedSDK from 'https://cdn.jsdelivr.net/npm/@onirix/embed-sdk@1.17.1/+esm';
import OnirixCameraModule from 'https://cdn.jsdelivr.net/npm/@onirix/camera-controls-module@1.2.1/+esm';

const embedSDK = new OnirixEmbedSDK();
embedSDK.connect();

const camera = new OnirixCameraModule(embedSDK);
```

Onirix Studio experiences can be loaded in two modes: AR and PREVIEW.
By default Onirix Camera Controls works in PREVIEW mode. If the experience is loaded in AR mode, the camera controls will be ignored.
The modes in which Onirix Camera Controls is enabled can be configured through the second parameter of the constructor:

```js
// Default: Only on PREVIEW mode
const camera = new OnirixCameraModule(embedSDK);

// Only on AR mode
const camera = new OnirixCameraModule(embedSDK, ['AR']);

// Both modes: AR and PREVIEW
const camera = new OnirixCameraModule(embedSDK, ['AR', 'PREVIEW']);

```

### Camera manipulation and animations

The camera can be moved using the `camera.animate(...)` function. This function receives the following parameters:

- `position_x`, `position_y`, and `position_z`: The coordinates the camera should move to. If any of them is
`undefined`, the camera will stay in place.
- `look_at_x`, `look_at_y`, and `look_at_z`: The coordinates the camera should look at. If any of them is `undefined`,
the camera will keep looking at the point it is currently looking at.
- `mode`: How to animate the transition to these positions. It is specified by a string with two components: an
interpolator (a component that describes the path the camera should take) and an easing function (a component that
describes the speed of the camera along said path). This string must be formated as `'<interpolator> <ease function>'`.
Alternatively, any of the components can be omitted. If the `mode` is optional (can be `undefined`).

There are two interpolators available:

- `linear`: The camera moves in a straight line.
- `spherical`: The camera moves following and arc around a center.

There are seven easing functions available:

- `linear-in-out`: The camera moves always at the same speed.
- `ease-in`: The camera starts slower.
- `ease-out`: The camera reduces its speed as it reaches its destination.
- `ease-in-out`: The camera starts accelerates at the begining and decelerates as it reaches the end.
- `bounce-in`: The camera anticipates the movement by going in the opposite direction for a short period of time.
- `bounce-out`: The camera overshoots the destination for a short period of time.
- `bounce-in-out`: The camera anticipates the movement by going in the opposite direction and overshoots the
destination for a short period of time.

Some examples of valid modes are `'linear'`, `'ease-in-out'`, `'spherical bounce-out'`.

- `time`: Length of the animation in seconds. If not provided it is assumed to be `0 `.
- `center_x`, `center_y`, and `center_z`: When using a spherical interpolator, the center of the arc. Otherwise it is
ignored.

### Controls
The camera controls are the components that describe how user interactions (touches, mouse clicks, key presses...) are
translated to camera movements around the scene. Camera controls are optional and you may use none if the camera cannot
be controlled by the user.

Currently, the camera module supports the following camera controls:

- **Orbit controls**: The camera moves around a point (the target). The user can zoom using the mouse wheel or by doing
pinch gestures, pan by dragging while pressing right mouse botton or sliding with two fingers, and cycle around by
dragging while pressing left mouse button or sliding with a single finger. They can be enabled using
`camera.enableOrbitControls()`.

The function `camera.disableControls()` can be used to disable the current camera controls.

#### Orbit controls
Some functions allow setting limits to the transformations that the user can perform:

- `camera.setOrbitControlsPanRange(min_x, max_x, min_y, max_y, min_z, max_z)` allows setting a minimum and maximum pan
value for each of the coordinate axes.
- `camera.setOrbitControlsRotateRange(min_x, max_x, min_y, max_y, min_z, max_z)` allows setting a minimum and maximum
rotation value for each of the coordiante axes. This rotation is measured as the angle from the camera to the axis.
- `camera.setOrbitControlsZoomRange(min, max)` allows setting a minimum and maximum zoom value.

Also, there is a function that allows setting the orbit target: `camera.setOrbitControlsTarget(x, y, z)`.

### Examples
#### Moving to a position on scene load
```js
import OnirixEmbedSDK from 'https://cdn.jsdelivr.net/npm/@onirix/embed-sdk@1.17.1/+esm';
import OnirixCameraModule from 'https://cdn.jsdelivr.net/npm/@onirix/camera-controls-module@1.2.1/+esm';

const embedSDK = new OnirixEmbedSDK();
embedSDK.connect();

const camera = new OnirixCameraModule(embedSDK);
camera.enableOrbitControls();

embedSDK.subscribe(OnirixEmbedSDK.Events.SCENE_LOAD_END, () => {
    camera.animateTo(
      -1, 0, 1,
      1, 0, -1,
      'spherical ease-in-out',
      5
    );
});
```

#### An orbit and zoom animation on scene load
```js
import OnirixEmbedSDK from 'https://cdn.jsdelivr.net/npm/@onirix/embed-sdk@1.17.1/+esm';
import OnirixCameraModule from 'https://cdn.jsdelivr.net/npm/@onirix/camera-controls-module@1.2.1/+esm';

const embedSDK = new OnirixEmbedSDK();
embedSDK.connect();

const camera = new OnirixCameraModule(embedSDK);
camera.enableOrbitControls();

embedSDK.subscribe(OnirixEmbedSDK.Events.SCENE_LOAD_END, async () => {
    await camera.animateTo(
        0, 0.5, -5,
        0, 0.5, 0
    );
    await camera.animateTo(
        0, 0.5, 5,
        0, 0.5, 0,
        'spherical ease-in-out',
        2,
        0, 0.5, 0
    );
    await camera.animateTo(
        0, 0.5, 2,
        0, 0.5, 0,
        'linear ease-in-out',
        2,
        0, 0.5, 0
    );
});
```

## Author

👤 **Onirix**

* Website: www.onirix.com
* Twitter: [@onirix](https://twitter.com/onirix)
* Github: [@onirix-ar](https://github.com/onirix-ar)
* LinkedIn: [@onirixar](https://linkedin.com/in/onirixar)
